/*
 * Copyright (c) 2009 The openGion Project.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND,
 * either express or implied. See the License for the specific language
 * governing permissions and limitations under the License.
 */
package org.opengion.fukurou.db;

import org.opengion.fukurou.util.ApplicationInfo;


/**
 * Transaction インターフェースを継承した、リアルタイムトランザクションクラスです。
 *
 * これは、通常のトランザクションクラスと異なり、トランザクション処理をしません。
 * つまり、トランザクション処理を行わないときに、メソッド等の共通的な使用ができるようにする、
 * Connection のラップクラスになります。
 * ただし、このクラスが生成されてから、クローズされるまでは、内部に保留した Connection は、
 * ずっと使い続けられますので、ConnectionFactory から取り出した Connection と同様に
 * トランザクション性は維持されます。
 * このクラスより作成された、Statement は、取り出し側で、きちんとクロース処理を
 *
 * このクラスでは、コミット、ロールバック、クローズ処理は、リアルタイムに行われます。
 *
 * @og.rev 5.1.9.0 (2010/08/01) 新規作成
 * @og.rev 5.3.8.0 (2011/08/01) クラスの内部構造変更
 *
 * @version  5.0
 * @author	 Kazuhiko Hasegawa
 * @since    JDK6.0,
 */
public class TransactionReal extends TransactionImpl {
//public class TransactionReal implements Transaction {
//	private final Connection conn ;
//	private final String     dbid ;
//	private		ApplicationInfo appInfo = null;
//	private		String     dbid = null;
//	private		Connection conn = null;			// 5.3.7.0 (2011/07/01) 一度だけ close するように変更します。

	/**
	 * ApplicationInfo を指定して作成する、コンストラクター
	 *
	 * このクラスは、トランザクション処理をしない場合に、従来の Connection の
	 * 代わりに使用することを想定したクラスのオブジェクトを作成します。
	 *
	 * @og.rev 5.3.7.0 (2011/07/01) dbidを引数から削除
	 * @og.rev 5.3.8.0 (2011/08/01) 親クラスを呼ぶように変更
	 *
	 * @param	appInfo	内部統制用のアクセス情報
	 */
//	public TransactionReal( final String dbid , final ApplicationInfo appInfo ) {
	public TransactionReal( final ApplicationInfo appInfo ) {
		super( appInfo );
//		this.appInfo = appInfo;
//		this.dbid = dbid;
//		this.conn = ConnectionFactory.connection( dbid,appInfo );
	}

	/**
	 * 呼ばれる都度、新しい Connection を返します。ただし、Connection 自身は、
	 * ConnectionFactory から取得するため、基本的にはキャッシュされたConnectionです。
	 *
	 * @og.rev 5.3.7.0 (2011/07/01) dbidチェック
	 * @og.rev 5.3.8.0 (2011/08/01) 親クラスのメソッドを呼ぶ為、廃止
	 *
	 * @param	dbid  接続先ID (このクラスでは無視されます)
	 *
	 * @return	指定のDBID に対応した、Connection オブジェクト
	 */
//	public Connection getConnection( final String dbid ) {
//		if( conn != null ) { close(); }
//
//		this.dbid = dbid;
//		this.conn = ConnectionFactory.connection( dbid,appInfo );
//		return conn ;
//	}

	/**
	 * コミット処理を行います。
	 *
	 * これは、呼ばれる都度、リアルタイムに処理します。
	 *
	 * @og.rev 5.3.8.0 (2011/08/01) 親クラスのメソッドを呼ぶ為、廃止
	 *
	 * @return 正常:true/異常:false
	 */
//	public boolean commit() {
//		return Closer.commit( conn );
//	}

	/**
	 * ロールバック処理を行います。
	 *
	 * これは、呼ばれる都度、リアルタイムに処理します。
	 *
	 * @og.rev 5.3.8.0 (2011/08/01) 親クラスのメソッドを呼ぶ為、廃止
	 *
	 * @return 正常:true/異常:false
	 */
//	public boolean rollback() {
//		return Closer.rollback( conn );
//	}

	/**
	 * トランザクションの、終了時処理を行います。
	 *
	 * @og.rev 5.3.8.0 (2011/08/01) 親クラスのメソッドを呼ぶ為、廃止
	 *
	 * @see #close( boolean )
	 *
	 * @return 正常:true/異常:false
	 */
//	public boolean close() {
//		return close( false );
//	}

	/**
	 * コネクションの、終了時処理を行います。
	 *
	 * 引数は、正常かどうかを判定するフラグです。異常の場合は、true をセットします。
	 * これは、ConnectionFactory のプールに戻すかどうかを判断するのに使われます。
	 * 一度でも、エラーが発生したコネクションは、破棄します。
	 * それ以外は、プールに戻します。
	 *
	 * @og.rev 5.3.7.0 (2011/07/01) close時に、コネクションを null 化しておく。
	 * @og.rev 5.3.8.0 (2011/08/01) 終了処理を行い、親クラスのrealClose() を呼ぶ。
	 *
	 * @param	errFlag  [true:エラー状態/false:通常]
	 *
	 * @return 正常:true/異常:false
	 */
	@Override
	public boolean close( final boolean errFlag ) {
//		if( errFlag ) { ConnectionFactory.remove( conn,dbid ); }	// 削除
//		else {			ConnectionFactory.close( conn,dbid );  }	// 返却

//		conn = null;	// 5.3.7.0 (2011/07/01)
//		return true;

		// 5.3.8.0 (2011/08/01) 終了処理を行い、親クラスのrealClose() を呼ぶ。
		super.close( errFlag );
		finish();
		realClose();

		return true;
	}
}
